import netgen.gui
import numpy as np
from ngsolve import *
from netgen.geom2d import *
import netgen as ng
from netgen.meshing import Element1D, Element2D, MeshPoint, FaceDescriptor
#from netgen.csg import Pnt




def MakeMesh_curved(L = 3, sgm = 0.5):
    geo = SplineGeometry()
    p1 =  geo.AppendPoint(0,0,hpref=1)
    p2 =  geo.AppendPoint(1.0,0,hpref=1)
    p3 =  geo.AppendPoint(1.0,1.0)
    p4 =  geo.AppendPoint(0,1,hpref=1)
    geo.Append(["line",p1,p2],hpref=1)
    geo.Append (["spline3", p2, p3, p4],hpref=1)
    geo.Append(["line",p4,p1],hpref=1)
        
    mesh = Mesh(geo.GenerateMesh(maxh=0.25, quad_dominated=True))
    mesh.RefineHP(L, factor=sgm)
    

    return mesh



def solveSing(eps = 0.1, p = 3, draw = 0):
    mesh = MakeMesh_curved(L = p, sgm = 0.25)

    fes = H1(mesh, order=p, dirichlet="default")
    fes.ndof  # number of unknowns in this space
    
    
    u = fes.TrialFunction() # symbolic object
    v = fes.TestFunction()   # symbolic object
    gfu = GridFunction(fes)  # solution 
    
    a = BilinearForm(fes, symmetric=True)
    b = BilinearForm(fes, symmetric=True)
    a += SymbolicBFI((eps**2)*grad(u)*grad(v))
    b += SymbolicBFI(u*v)
    a.Assemble()
    b.Assemble()
    
    f = LinearForm(fes)
    f += SymbolicLFI(1.0*v)
    f.Assemble()
    

    mstar = b.mat.CreateMatrix()

    mstar.AsVector().data = b.mat.AsVector() + a.mat.AsVector()
    tmp = mstar.Inverse(freedofs=fes.FreeDofs()) * f.vec
    gfu.vec.data = tmp

    if (draw > 0):
        Draw(gfu)


    #eval energy
    M = f.vec.size
    enrg = 0.0;
    for j in range(M):
        enrg += f.vec[j]*gfu.vec[j]
        
    return enrg

P = 10
res = np.zeros(P)

for j in range(P):
    res[j] = solveSing(eps=0.01,p=j+1);

for j in range(P):
    print(res[j])
# to plot error, plot sqrt(res(end)-res(1:end-1)) 
# against polynomial degree
